@@ -4,6 +4,7 @@ package MY::TypeLib::MouseX;
 use 5.006;
 use strict;
 use warnings;
+use MY::Helpers;
 # within Mouse, so we put it in a separate package from SampleMouseXTypes.
 
 use Scalar::Util qw(looks_like_number);
@@ -18,7 +19,7 @@ BEGIN {
     subtype MediumInteger,
         as Int,
         where { $_ >= 10 and $_ < 20 },
-        message { ($_ // 'undef') . ' is not an integer on [10,19]' };
+        message { _dor . ' is not an integer on [10,19]' };
 
     # Sanity check
     my $av = eval { MediumInteger->can('assert_valid') };
